@@ -11,11 +11,13 @@ from tryton.gui.window import Window
 from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
+from tryton.gui.window.revision import Revision
 from tryton.signal_event import SignalEvent
 from tryton.common import message, sur, sur_3b, COLOR_SCHEMES, timezoned_date
 import tryton.common as common
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
+from tryton.common.datetime_strftime import datetime_strftime
 from tryton import plugins
 
 from tabcontent import TabContent
@@ -61,6 +63,7 @@ class Form(SignalEvent, TabContent):
         (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
         (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
         (_('View _Logs...'), None, 'sig_logs', None),
+        (_('Show revisions...'), 'tryton-clock', 'revision', None),
         (None,) * 4,
         (_('_Close Tab'), 'tryton-close', 'sig_win_close',
             '<tryton>/Form/Close'),
@@ -85,9 +88,9 @@ class Form(SignalEvent, TabContent):
             '<tryton>/Form/Import Data'),
     ]
 
-    def __init__(self, model, res_id=False, domain=None, mode=None,
+    def __init__(self, model, res_id=False, domain=None, order=None, mode=None,
             view_ids=None, context=None, name=False, limit=None,
-            auto_refresh=False, search_value=None, tab_domain=None):
+            search_value=None, tab_domain=None):
         super(Form, self).__init__()
 
         if not mode:
@@ -96,22 +99,18 @@ class Form(SignalEvent, TabContent):
             domain = []
         if view_ids is None:
             view_ids = []
-        if context is None:
-            context = {}
 
         self.model = model
         self.res_id = res_id
         self.domain = domain
         self.mode = mode
         self.context = context
-        self.auto_refresh = auto_refresh
         self.view_ids = view_ids
         self.dialogs = []
 
-        self.screen = Screen(self.model, mode=mode, context=self.context,
-            view_ids=view_ids, domain=domain, limit=limit,
-            readonly=bool(auto_refresh), search_value=search_value,
-            tab_domain=tab_domain)
+        self.screen = Screen(self.model, mode=mode, context=context,
+            view_ids=view_ids, domain=domain, limit=limit, order=order,
+            search_value=search_value, tab_domain=tab_domain)
         self.screen.widget.show()
 
         if not name:
@@ -119,8 +118,22 @@ class Form(SignalEvent, TabContent):
         else:
             self.name = name
 
+        if self.model not in common.MODELHISTORY:
+            self.menu_def = self.menu_def[:]
+            # Remove callback to revision
+            self.menu_def[11] = (self.menu_def[11][:2] + (None,)
+                + self.menu_def[11][3:])
+
         self.create_tabcontent()
 
+        self.url_entry = url_entry = gtk.Entry()
+        url_entry.show()
+        url_entry.set_editable(False)
+        style = url_entry.get_style()
+        url_entry.modify_bg(gtk.STATE_ACTIVE,
+            style.bg[gtk.STATE_INSENSITIVE])
+        self.widget.pack_start(url_entry, False, False)
+
         access = common.MODELACCESS[self.model]
         for button, access_type in (
                 ('new', 'create'),
@@ -130,6 +143,7 @@ class Form(SignalEvent, TabContent):
 
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
+
         self.screen.signal_connect(self, 'record-modified',
             lambda *a: gobject.idle_add(self._record_modified, *a))
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
@@ -147,13 +161,12 @@ class Form(SignalEvent, TabContent):
                     in ('tree', 'graph', 'calendar'):
                 self.screen.search_filter()
 
-        if auto_refresh and int(auto_refresh):
-            gobject.timeout_add(int(auto_refresh) * 1000, self.sig_reload)
+        self.update_revision()
 
     def get_toolbars(self):
         try:
             return RPCExecute('model', self.model, 'view_toolbar_get',
-                context=self.context)
+                context=self.screen.context)
         except RPCException:
             return {}
 
@@ -170,21 +183,13 @@ class Form(SignalEvent, TabContent):
             and self.domain == value.domain
             and self.mode == value.mode
             and self.view_ids == value.view_ids
-            and self.context == value.context
+            and self.screen.context == value.screen.context
             and self.name == value.name
             and self.screen.limit == value.screen.limit
-            and self.auto_refresh == value.auto_refresh
             and self.screen.search_value == value.screen.search_value)
 
     def destroy(self):
         self.screen.destroy()
-        self.screen = None
-        self.widget = None
-        #self.scrolledwindow.destroy()
-        #self.scrolledwindow = None
-
-    def sel_ids_get(self):
-        return self.screen.sel_ids_get()
 
     def ids_get(self):
         return self.screen.ids_get()
@@ -193,10 +198,10 @@ class Form(SignalEvent, TabContent):
         return self.screen.id_get()
 
     def sig_attach(self, widget=None):
-        record_id = self.id_get()
-        if record_id is False or record_id < 0:
+        record = self.screen.current_record
+        if not record or record.id < 0:
             return
-        Attachment(self.model, record_id,
+        Attachment(record,
             lambda: self.update_attachment_count(reload=True))
 
     def update_attachment_count(self, reload=False):
@@ -239,7 +244,7 @@ class Form(SignalEvent, TabContent):
 
         try:
             res = RPCExecute('model', self.model, 'read', [obj_id],
-                [x[0] for x in fields], context=self.context)
+                [x[0] for x in fields], context=self.screen.context)
         except RPCException:
             return
         message_str = ''
@@ -256,6 +261,42 @@ class Form(SignalEvent, TabContent):
         message(message_str)
         return True
 
+    def revision(self, widget=None):
+        if not self.modified_save():
+            return
+        current_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
+        try:
+            revisions = RPCExecute('model', self.model, 'history_revisions',
+                [r.id for r in self.screen.selected_records])
+        except RPCException:
+            return
+        revision = self.screen.context.get('_datetime')
+        revision = Revision(revisions, revision).run()
+        if revision != self.screen.context.get('_datetime'):
+            self.screen.clear()
+            # Update root group context that will be propagated
+            self.screen.group._context['_datetime'] = revision
+            if self.screen.current_view.view_type != 'form':
+                self.screen.search_filter(
+                    self.screen.screen_container.get_text())
+            else:
+                # Test if record exist in revisions
+                self.screen.load([current_id])
+            self.screen.display(set_cursor=True)
+            self.update_revision()
+
+    def update_revision(self):
+        revision = self.screen.context.get('_datetime')
+        if revision:
+            format_ = date_format() + ' %H:%M:%S.%f'
+            revision = datetime_strftime(revision, format_)
+            self.title.set_label('%s @ %s' % (self.name, revision))
+        else:
+            self.title.set_label(self.name)
+        for button in ('new', 'save'):
+            self.buttons[button].props.sensitive = not revision
+
     def sig_remove(self, widget=None):
         if not common.MODELACCESS[self.model]['delete']:
             return
@@ -275,7 +316,7 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        WinImport(self.model)
+        WinImport(self.model, self.screen.context)
 
     def sig_save_as(self, widget=None):
         while self.screen.view_to_load:
@@ -283,7 +324,7 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        WinExport(self.model, self.ids_get(), context=self.context)
+        WinExport(self.model, self.ids_get(), context=self.screen.context)
 
     def sig_new(self, widget=None, autosave=True):
         if not common.MODELACCESS[self.model]['create']:
@@ -300,13 +341,7 @@ class Form(SignalEvent, TabContent):
             return
         if not self.modified_save():
             return
-        res_ids = self.sel_ids_get()
-        try:
-            new_ids = RPCExecute('model', self.model, 'copy', res_ids, {},
-                context=self.context)
-        except RPCException:
-            return
-        self.screen.load(new_ids)
+        self.screen.copy()
         self.message_info(_('Working now on the duplicated record(s)!'),
             'green')
 
@@ -335,24 +370,17 @@ class Form(SignalEvent, TabContent):
         self.activate_save()
 
     def sig_reload(self, test_modified=True):
-        if not hasattr(self, 'screen'):
-            return False
-        if test_modified and self.screen.modified():
-            res = sur_3b(_('This record has been modified\n'
-                    'do you want to save it ?'))
-            if res == 'ok':
-                self.sig_save(None)
-            elif res == 'ko':
-                pass
-            else:
+        if test_modified and not self.modified_save():
                 return False
         self.screen.cancel_current()
         set_cursor = False
+        self.screen.save_tree_state(store=False)
+        record_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
         if self.screen.current_view.view_type != 'form':
-            obj_id = self.id_get()
             self.screen.search_filter(self.screen.screen_container.get_text())
             for record in self.screen.group:
-                if record.id == obj_id:
+                if record.id == record_id:
                     self.screen.current_record = record
                     set_cursor = True
                     break
@@ -420,7 +448,7 @@ class Form(SignalEvent, TabContent):
         name = '_'
         if signal_data[0]:
             name = str(signal_data[0])
-        for button_id in ('print', 'action', 'relate', 'email', 'open', 'save',
+        for button_id in ('print', 'relate', 'email', 'open', 'save',
                 'attach'):
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
@@ -435,11 +463,12 @@ class Form(SignalEvent, TabContent):
         self.status_label.set_text(msg)
         self.message_info('')
         self.activate_save()
+        self.url_entry.set_text(self.screen.get_url())
 
     def _record_modified(self, screen, signal_data):
         # As it is called via idle_add, the form could have been destroyed in
         # the meantime.
-        if screen == self.screen:
+        if self.widget_get().props.window:
             self.activate_save()
 
     def _record_saved(self, screen, signal_data):
@@ -447,6 +476,7 @@ class Form(SignalEvent, TabContent):
         self.update_attachment_count()
 
     def modified_save(self):
+        self.screen.save_tree_state()
         self.screen.current_view.set_value()
         if self.screen.modified():
             value = sur_3b(
@@ -468,15 +498,16 @@ class Form(SignalEvent, TabContent):
         action = action.copy()
         if not self.screen.save_current():
             return
-        record_id = self.screen.id_get()
-        record_ids = self.screen.sel_ids_get()
+        record_id = (self.screen.current_record.id
+            if self.screen.current_record else None)
+        record_ids = [r.id for r in self.screen.selected_records]
         action = Action.evaluate(action, atype, self.screen.current_record)
         data = {
             'model': self.screen.model_name,
             'id': record_id,
             'ids': record_ids,
         }
-        Action._exec_action(action, data, self.context)
+        Action._exec_action(action, data, self.screen.context)
 
     def activate_save(self):
         self.buttons['save'].props.sensitive = self.screen.modified()
@@ -487,6 +518,13 @@ class Form(SignalEvent, TabContent):
     def create_toolbar(self, toolbars):
         gtktoolbar = super(Form, self).create_toolbar(toolbars)
 
+        attach_btn = self.buttons['attach']
+        attach_btn.drag_dest_set(gtk.DEST_DEFAULT_ALL, [
+                ('text/uri-list', 0, 0),
+                ], gtk.gdk.ACTION_MOVE)
+        attach_btn.connect('drag_data_received',
+            self.attach_drag_data_received)
+
         iconstock = {
             'print': 'tryton-print',
             'action': 'tryton-executable',
@@ -510,7 +548,9 @@ class Form(SignalEvent, TabContent):
                 tbutton.connect('toggled', self.action_popup)
                 self.tooltips.set_tip(tbutton, tooltip)
                 self.buttons[special_action] = tbutton
-                tbutton._can_be_sensitive = bool(tbutton._menu.get_children())
+                if action_type != 'action':
+                    tbutton._can_be_sensitive = bool(
+                        tbutton._menu.get_children())
             else:
                 tbutton = gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
@@ -521,6 +561,9 @@ class Form(SignalEvent, TabContent):
         menu = gtk.Menu()
         menu.connect('deactivate', self._popup_menu_hide, widget)
 
+        if keyword == 'action':
+            widget.connect('toggled', self._update_action_popup, menu)
+
         for action in actions:
             new_action = action.copy()
             if special_action == 'print':
@@ -535,18 +578,6 @@ class Form(SignalEvent, TabContent):
             menuitem.connect('activate', self._popup_menu_selected, widget,
                 new_action, keyword)
             menu.add(menuitem)
-        if keyword == 'action':
-            menu.add(gtk.SeparatorMenuItem())
-            for plugin in plugins.MODULES:
-                for name, func in plugin.get_plugins(self.model):
-                    menuitem = gtk.MenuItem('_' + name)
-                    menuitem.set_use_underline(True)
-                    menuitem.connect('activate', lambda m, func: func({
-                                'model': self.model,
-                                'ids': self.id_get(),
-                                'id': self.id_get(),
-                                }), func)
-                    menu.add(menuitem)
         return menu
 
     def _popup_menu_selected(self, menuitem, togglebutton, action, keyword):
@@ -562,6 +593,52 @@ class Form(SignalEvent, TabContent):
     def _popup_menu_hide(self, menuitem, togglebutton):
         togglebutton.props.active = False
 
+    def _update_action_popup(self, tbutton, menu):
+        for item in menu.get_children():
+            if (getattr(item, '_update_action', False)
+                    or isinstance(item, gtk.SeparatorMenuItem)):
+                menu.remove(item)
+
+        buttons = self.screen.get_buttons()
+        if buttons:
+            menu.add(gtk.SeparatorMenuItem())
+        for button in buttons:
+            menuitem = gtk.ImageMenuItem(button.attrs.get('icon'))
+            menuitem.set_label('_' + button.attrs.get('string', _('Unknown')))
+            menuitem.set_use_underline(True)
+            menuitem.connect('activate',
+                lambda m, attrs: self.screen.button(attrs), button.attrs)
+            menuitem._update_action = True
+            menu.add(menuitem)
+
+        menu.add(gtk.SeparatorMenuItem())
+        for plugin in plugins.MODULES:
+            for name, func in plugin.get_plugins(self.model):
+                menuitem = gtk.MenuItem('_' + name)
+                menuitem.set_use_underline(True)
+                menuitem.connect('activate', lambda m, func: func({
+                            'model': self.model,
+                            'ids': [r.id
+                                for r in self.screen.selected_records],
+                            'id': self.id_get(),
+                            }), func)
+                menuitem._update_action = True
+                menu.add(menuitem)
+
     def set_cursor(self):
         if self.screen:
             self.screen.set_cursor(reset_view=False)
+
+    def attach_drag_data_received(self, widget, context, x, y, selection, info,
+            timestamp):
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            return
+        win_attach = Attachment(record,
+            lambda: self.update_attachment_count(reload=True))
+        if info == 0:
+            for uri in selection.data.splitlines():
+                # Win32 cut&paste terminates the list with a NULL character
+                if not uri or uri == '\0':
+                    continue
+                win_attach.add_uri(uri)
