@@ -1,16 +1,18 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 import os
+from itertools import chain
 import tempfile
 import locale
 from tryton.common import datetime_strftime, \
-        domain_inversion, eval_domain, localize_domain, unlocalize_domain, \
-        merge, inverse_leaf, EvalEnvironment
+        domain_inversion, eval_domain, localize_domain, \
+        merge, inverse_leaf, concat, simplify, EvalEnvironment
 import tryton.common as common
 import time
 import datetime
 import decimal
 from decimal import Decimal
+import math
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
 
@@ -22,15 +24,11 @@ class Field(object):
     set: save the value from the server
     set_client: save the value from the widget
     '''
+    _default = None
 
-    def __new__(cls, ctype):
-        klass = TYPES.get(ctype, CharField)
-        return klass
-
-
-class CharField(object):
-
-    _default = ''
+    @staticmethod
+    def get_field(ctype):
+        return TYPES.get(ctype, CharField)
 
     def __init__(self, attrs):
         self.attrs = attrs
@@ -48,7 +46,7 @@ class CharField(object):
         screen_domain = domain_inversion(record.group.domain4inversion,
             self.name, EvalEnvironment(record))
         if isinstance(screen_domain, bool) and not screen_domain:
-            screen_domain = [('id', '=', False)]
+            screen_domain = [('id', '=', None)]
         elif isinstance(screen_domain, bool) and screen_domain:
             screen_domain = []
         attr_domain = record.expr_eval(self.attrs.get('domain', []))
@@ -56,15 +54,10 @@ class CharField(object):
 
     def domain_get(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return localize_domain(screen_domain) + attr_domain
+        return concat(localize_domain(screen_domain), attr_domain)
 
     def validation_domains(self, record):
-        screen_domain, attr_domain = self.domains_get(record)
-        if attr_domain:
-            return (screen_domain, screen_domain +
-                unlocalize_domain(attr_domain, self.name))
-        else:
-            return screen_domain, screen_domain
+        return concat(*self.domains_get(record))
 
     def context_get(self, record):
         context = record.context_get().copy()
@@ -76,7 +69,7 @@ class CharField(object):
     def check_required(self, record):
         state_attrs = self.get_state_attrs(record)
         if bool(int(state_attrs.get('required') or 0)):
-            if (not self.get(record)
+            if (not self.get_eval(record)
                     and not bool(int(state_attrs.get('readonly') or 0))):
                 return False
         return True
@@ -86,38 +79,43 @@ class CharField(object):
             return True
         res = True
         self.get_state_attrs(record)['domain_readonly'] = False
-        inverted_domain, domain = self.validation_domains(record)
+        domain = simplify(self.validation_domains(record))
         if not softvalidation:
             res = res and self.check_required(record)
         if isinstance(domain, bool):
             res = res and domain
-        elif domain == [('id', '=', False)]:
+        elif domain == [('id', '=', None)]:
             res = False
         else:
-            if (isinstance(inverted_domain, list)
-                    and len(inverted_domain) == 1
-                    and inverted_domain[0][1] == '='):
+            if (isinstance(domain, list)
+                    and len(domain) == 1
+                    and domain[0][1] == '='):
                 # If the inverted domain is so constraint that only one value
                 # is possible we should use it. But we must also pay attention
                 # to the fact that the original domain might be a 'OR' domain
                 # and thus not preventing the modification of fields.
-                leftpart, _, value = inverted_domain[0][:3]
+                leftpart, _, value = domain[0][:3]
                 if value is False:
                     # XXX to remove once server domains are fixed
                     value = None
                 setdefault = True
+                if record.group.domain:
+                    original_domain = merge(record.group.domain)
+                else:
+                    original_domain = merge(domain)
+                domain_readonly = original_domain[0] == 'AND'
                 if '.' in leftpart:
                     recordpart, localpart = leftpart.split('.', 1)
-                    original_domain = merge(record.group.domain)
                     constraintfields = set()
-                    if original_domain[0] == 'AND':
+                    if domain_readonly:
                         for leaf in localize_domain(original_domain[1:]):
                             constraintfields.add(leaf[0])
                     if localpart != 'id' or recordpart not in constraintfields:
                         setdefault = False
                 if setdefault:
                     self.set_client(record, value)
-                    self.get_state_attrs(record)['domain_readonly'] = True
+                    self.get_state_attrs(record)['domain_readonly'] = (
+                        domain_readonly)
             res = res and eval_domain(domain, EvalEnvironment(record))
         self.get_state_attrs(record)['valid'] = res
         return res
@@ -126,7 +124,7 @@ class CharField(object):
         record.value[self.name] = value
 
     def get(self, record):
-        return record.value.get(self.name) or self._default
+        return record.value.get(self.name, self._default)
 
     def get_eval(self, record):
         return self.get(record)
@@ -149,7 +147,7 @@ class CharField(object):
             record.signal('record-changed')
 
     def get_client(self, record):
-        return record.value.get(self.name) or self._default
+        return self.get(record)
 
     def set_default(self, record, value):
         self.set(record, value)
@@ -183,7 +181,14 @@ class CharField(object):
         return {}
 
 
-class SelectionField(CharField):
+class CharField(Field):
+    _default = ''
+
+    def get(self, record):
+        return super(CharField, self).get(record) or self._default
+
+
+class SelectionField(Field):
 
     _default = None
 
@@ -191,18 +196,19 @@ class SelectionField(CharField):
         return record.value.get(self.name)
 
 
-class DateTimeField(CharField):
+class DateTimeField(Field):
 
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if not isinstance(value, datetime.datetime):
+        if isinstance(value, basestring):
             try:
                 value = datetime.datetime(*time.strptime(value,
                         date_format() + ' ' + self.time_format(record))[:6])
-                value = common.untimezoned_date(value)
             except ValueError:
                 value = self._default
+        if value:
+            value = common.untimezoned_date(value)
         super(DateTimeField, self).set_client(record, value,
             force_change=force_change)
 
@@ -218,17 +224,20 @@ class DateTimeField(CharField):
         return record.expr_eval(self.attrs['format'])
 
 
-class DateField(CharField):
+class DateField(Field):
 
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if not isinstance(value, datetime.date):
+        if isinstance(value, basestring):
             try:
                 value = datetime.date(*time.strptime(value,
                         date_format())[:3])
             except ValueError:
                 value = self._default
+        elif isinstance(value, datetime.datetime):
+            assert(value.time() == datetime.time())
+            value = value.date()
         super(DateField, self).set_client(record, value,
             force_change=force_change)
 
@@ -239,17 +248,19 @@ class DateField(CharField):
         return ''
 
 
-class TimeField(CharField):
+class TimeField(Field):
 
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if not isinstance(value, datetime.time):
+        if isinstance(value, basestring):
             try:
                 value = datetime.time(*time.strptime(value,
                         self.time_format(record))[3:6])
             except ValueError:
-                value = None
+                value = self._default
+        elif isinstance(value, datetime.datetime):
+            value = value.time()
         super(TimeField, self).set_client(record, value,
             force_change=force_change)
 
@@ -263,8 +274,9 @@ class TimeField(CharField):
         return record.expr_eval(self.attrs['format'])
 
 
-class NumberField(CharField):
+class FloatField(Field):
     _default = None
+    default_digits = (16, 2)
 
     def check_required(self, record):
         state_attrs = self.get_state_attrs(record)
@@ -277,74 +289,72 @@ class NumberField(CharField):
     def get(self, record):
         return record.value.get(self.name, self._default)
 
-    def digits(self, record):
-        default = (16, 2)
-        return tuple(y if x is None else x for x, y in zip(
-                record.expr_eval(self.attrs.get('digits', default)), default))
+    def digits(self, record, factor=1):
+        digits = tuple(y if x is None else x for x, y in zip(
+                record.expr_eval(
+                    self.attrs.get('digits', self.default_digits)),
+                self.default_digits))
+        shift = int(round(math.log(factor, 10)))
+        return (digits[0] + shift, digits[1] - shift)
 
+    def convert(self, value):
+        try:
+            return locale.atof(value)
+        except ValueError:
+            return self._default
 
-class FloatField(NumberField):
-
-    def set_client(self, record, value, force_change=False):
+    def set_client(self, record, value, force_change=False, factor=1):
         if isinstance(value, basestring):
-            try:
-                value = locale.atof(value)
-            except ValueError:
-                value = self._default
+            value = self.convert(value)
+        if value is not None:
+            value /= factor
         super(FloatField, self).set_client(record, value,
             force_change=force_change)
 
-    def get_client(self, record):
+    def get_client(self, record, factor=1):
         value = record.value.get(self.name)
         if value is not None:
-            digits = self.digits(record)
-            return locale.format('%.*f', (digits[1], value), True)
+            digit = self.digits(record, factor=factor)[1]
+            return locale.format('%.*f', (digit, value * factor), True)
         else:
             return ''
 
 
-class NumericField(NumberField):
+class NumericField(FloatField):
 
-    def set_client(self, record, value, force_change=False):
-        if isinstance(value, basestring):
-            try:
-                value = locale.atof(value, Decimal)
-            except decimal.InvalidOperation:
-                value = self._default
-        super(NumericField, self).set_client(record, value,
-            force_change=force_change)
+    def convert(self, value):
+        try:
+            return locale.atof(value, Decimal)
+        except decimal.InvalidOperation:
+            return self._default
 
-    def get_client(self, record):
-        value = record.value.get(self.name)
-        if value is not None:
-            digits = self.digits(record)
-            return locale.format('%.*f', (digits[1], value), True)
-        else:
-            return ''
+    def set_client(self, record, value, force_change=False, factor=1):
+        return super(NumericField, self).set_client(record, value,
+            force_change=force_change, factor=Decimal(str(factor)))
+
+    def get_client(self, record, factor=1):
+        return super(NumericField, self).get_client(record,
+            factor=Decimal(str(factor)))
 
 
-class IntegerField(NumberField):
+class IntegerField(FloatField):
+    default_digits = (16, 0)
 
-    def set_client(self, record, value):
-        if isinstance(value, basestring):
-            try:
-                value = locale.atoi(value)
-            except ValueError:
-                value = self._default
-        super(IntegerField, self).set_client(record, value)
+    def convert(self, value):
+        try:
+            return locale.atoi(value)
+        except ValueError:
+            return self._default
 
-    def get_client(self, record):
-        value = record.value.get(self.name)
-        if value is not None:
-            return locale.format('%d', value, True)
-        else:
-            return ''
+    def set_client(self, record, value, force_change=False, factor=1):
+        return super(IntegerField, self).set_client(record, value,
+            force_change=force_change, factor=int(factor))
 
-    def digits(self, record):
-        return (16, 0)
+    def get_client(self, record, factor=1):
+        return super(IntegerField, self).get_client(record, factor=int(factor))
 
 
-class BooleanField(CharField):
+class BooleanField(Field):
 
     _default = False
 
@@ -360,7 +370,7 @@ class BooleanField(CharField):
         return bool(record.value.get(self.name))
 
 
-class M2OField(CharField):
+class M2OField(Field):
     '''
     internal = (id, name)
     '''
@@ -408,7 +418,7 @@ class M2OField(CharField):
         if not rec_name and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
-                    [value], ['rec_name'], main_iteration=False)
+                    [value], ['rec_name'])
             except RPCException:
                 return False
             rec_name = result['rec_name'] or ''
@@ -429,12 +439,12 @@ class M2OField(CharField):
 
     def validation_domains(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return screen_domain, screen_domain
+        return screen_domain
 
     def domain_get(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return (localize_domain(inverse_leaf(screen_domain), self.name)
-            + attr_domain)
+        return concat(localize_domain(inverse_leaf(screen_domain), self.name),
+            attr_domain)
 
     def get_state_attrs(self, record):
         result = super(M2OField, self).get_state_attrs(record)
@@ -449,7 +459,7 @@ class O2OField(M2OField):
     pass
 
 
-class O2MField(CharField):
+class O2MField(Field):
     '''
     internal = Group of the related objects
     '''
@@ -468,10 +478,12 @@ class O2MField(CharField):
     def _group_changed(self, group, record):
         if not record.parent:
             return
-        record.parent.modified_fields.setdefault(self.name)
-        self.sig_changed(record.parent)
-        record.parent.validate(softvalidation=True)
-        record.parent.signal('record-changed')
+        # Store parent as it could be removed by validation
+        parent = record.parent
+        parent.modified_fields.setdefault(self.name)
+        self.sig_changed(parent)
+        parent.validate(softvalidation=True)
+        parent.signal('record-changed')
 
     def _group_list_changed(self, group, signal):
         if group.model_name == group.parent.model_name:
@@ -486,25 +498,29 @@ class O2MField(CharField):
             return
         record.parent.signal('record-modified')
 
-    def _set_default_value(self, record):
+    def _set_default_value(self, record, fields=None):
         if record.value.get(self.name) is not None:
             return
         from group import Group
         parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {},
+        fields = fields or {}
+        group = Group(self.attrs['relation'], fields,
                 parent=record,
                 parent_name=parent_name,
                 child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
-        if record.model_name == self.attrs['relation']:
+        if not fields and record.model_name == self.attrs['relation']:
             group.fields = record.group.fields
+        record.value[self.name] = group
+        self._connect_value(group)
+
+    def _connect_value(self, group):
         group.signal_connect(group, 'group-changed', self._group_changed)
         group.signal_connect(group, 'group-list-changed',
             self._group_list_changed)
         group.signal_connect(group, 'group-cleared', self._group_cleared)
         group.signal_connect(group, 'record-modified', self._record_modified)
-        record.value[self.name] = group
 
     def get_client(self, record):
         self._set_default_value(record)
@@ -515,9 +531,11 @@ class O2MField(CharField):
             return []
         record_removed = record.value[self.name].record_removed
         record_deleted = record.value[self.name].record_deleted
-        result = [('add', [])]
+        result = []
         parent_name = self.attrs.get('relation_field', '')
+        to_add = []
         to_create = []
+        to_write = []
         for record2 in record.value[self.name]:
             if record2 in record_removed or record2 in record_deleted:
                 continue
@@ -525,18 +543,20 @@ class O2MField(CharField):
                 values = record2.get()
                 values.pop(parent_name, None)
                 if record2.modified and values:
-                    result.append(('write', [record2.id], values))
-                result[0][1].append(record2.id)
+                    to_write.extend(([record2.id], values))
+                to_add.append(record2.id)
             else:
                 values = record2.get()
                 values.pop(parent_name, None)
                 to_create.append(values)
+        if to_add:
+            result.append(('add', to_add))
         if to_create:
             result.append(('create', to_create))
-        if not result[0][1]:
-            del result[0]
+        if to_write:
+            result.append(('write',) + tuple(to_write))
         if record_removed:
-            result.append(('unlink', [x.id for x in record_removed]))
+            result.append(('remove', [x.id for x in record_removed]))
         if record_deleted:
             result.append(('delete', [x.id for x in record_deleted]))
         return result
@@ -570,13 +590,42 @@ class O2MField(CharField):
         return result
 
     def _set_value(self, record, value, default=False):
-        from group import Group
-
+        self._set_default_value(record)
+        group = record.value[self.name]
         if not value or (len(value) and isinstance(value[0], (int, long))):
             mode = 'list ids'
         else:
             mode = 'list values'
 
+        if mode == 'list values' and len(value):
+            context = self.context_get(record)
+            field_names = set(f for v in value for f in v
+                if f not in group.fields)
+            if field_names:
+                try:
+                    fields = RPCExecute('model', self.attrs['relation'],
+                        'fields_get', list(field_names), context=context)
+                except RPCException:
+                    return
+                group.load_fields(fields)
+
+        if mode == 'list ids':
+            for old_record in group:
+                if old_record.id not in value:
+                    group.remove(old_record, remove=True, signal=False)
+            group.load(value)
+        else:
+            for vals in value:
+                new_record = record.value[self.name].new(default=False)
+                if default:
+                    new_record.set_default(vals)
+                    group.add(new_record)
+                else:
+                    new_record.id *= -1  # Don't consider record as unsaved
+                    new_record.set(vals)
+                    group.append(new_record)
+
+    def set(self, record, value, _default=False):
         group = record.value.get(self.name)
         fields = {}
         if group is not None:
@@ -589,60 +638,39 @@ class O2MField(CharField):
         if fields:
             fields = dict((fname, field.attrs)
                 for fname, field in fields.iteritems())
-        if mode == 'list values' and len(value):
-            context = self.context_get(record)
-            field_names = set(f for v in value for f in v if f not in fields)
-            if field_names:
-                try:
-                    fields.update(RPCExecute('model', self.attrs['relation'],
-                            'fields_get', list(field_names),
-                            main_iteration=False, context=context))
-                except RPCException:
-                    return
 
-        parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], fields,
-            parent=record, parent_name=parent_name,
-            child_name=self.name,
-            context=self.context,
-            parent_datetime_field=self.attrs.get('datetime_field'))
-        record.value[self.name] = group
-        if mode == 'list ids':
-            group.load(value)
-        else:
-            for vals in value:
-                new_record = record.value[self.name].new(default=False)
-                if default:
-                    new_record.set_default(vals)
-                    group.add(new_record)
-                else:
-                    new_record.id *= -1  # Don't consider record as unsaved
-                    new_record.set(vals)
-                    group.append(new_record)
-        group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed',
-            self._group_list_changed)
-        group.signal_connect(group, 'group-cleared', self._group_cleared)
-        group.signal_connect(group, 'record-modified', self._record_modified)
-        return group
+        record.value[self.name] = None
+        self._set_default_value(record, fields=fields)
+        group = record.value[self.name]
 
-    def set(self, record, value):
-        self._set_value(record, value, default=False)
+        group.signal_unconnect(group)
+        self._set_value(record, value, default=_default)
+        self._connect_value(group)
 
     def set_client(self, record, value, force_change=False):
-        pass
+        # domain inversion could try to set id as value
+        if isinstance(value, (int, long)):
+            value = [value]
+
+        previous_ids = self.get_eval(record)
+        self._set_value(record, value)
+        # The order of the ids is not significant
+        if set(previous_ids) != set(value):
+            record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            self.sig_changed(record)
+            record.validate(softvalidation=True)
+            record.signal('record-changed')
+        elif force_change:
+            self.sig_changed(record)
+            record.validate(softvalidation=True)
+            record.signal('record-changed')
 
     def set_default(self, record, value):
-        previous_group = record.value.get(self.name)
-        group = self._set_value(record, value, default=True)
-        if previous_group:
-            group.record_deleted.extend(x for x in previous_group if x.id >= 0)
-            group.record_deleted.extend(previous_group.record_deleted)
-            group.record_removed.extend(previous_group.record_removed)
+        self.set(record, value, _default=True)
         record.modified_fields.setdefault(self.name)
 
     def set_on_change(self, record, value):
-        self._set_default_value(record)
         if isinstance(value, (list, tuple)):
             self._set_value(record, value)
             record.modified_fields.setdefault(self.name)
@@ -652,14 +680,14 @@ class O2MField(CharField):
         if value and (value.get('add') or value.get('update')):
             context = self.context_get(record)
             fields = record.value[self.name].fields
-            field_names = set(f for v in (
-                    value.get('add', []) + value.get('update', []))
+            values = chain(value.get('update', []),
+                (d for _, d in value.get('add', [])))
+            field_names = set(f for v in values
                 for f in v if f not in fields and f != 'id')
             if field_names:
                 try:
                     fields = RPCExecute('model', self.attrs['relation'],
-                        'fields_get', list(field_names), main_iteration=False,
-                        context=context)
+                        'fields_get', list(field_names), context=context)
                 except RPCException:
                     return False
             else:
@@ -680,9 +708,9 @@ class O2MField(CharField):
 
         if value and (value.get('add') or value.get('update', [])):
             record.value[self.name].add_fields(fields, signal=False)
-            for vals in value.get('add', []):
+            for index, vals in value.get('add', []):
                 new_record = record.value[self.name].new(default=False)
-                record.value[self.name].add(new_record)
+                record.value[self.name].add(new_record, index)
                 new_record.set_on_change(vals)
 
             for vals in value.get('update', []):
@@ -695,7 +723,7 @@ class O2MField(CharField):
 
     def validation_domains(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return screen_domain, screen_domain
+        return screen_domain
 
     def validate(self, record, softvalidation=False):
         if self.attrs.get('readonly'):
@@ -725,46 +753,17 @@ class O2MField(CharField):
 
     def domain_get(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return localize_domain(inverse_leaf(screen_domain)) + attr_domain
+        return concat(localize_domain(inverse_leaf(screen_domain), self.name),
+            attr_domain)
 
 
 class M2MField(O2MField):
 
-    def set(self, record, value):
-        from group import Group
-        group = record.value.get(self.name)
-        fields = {}
-        if group is not None:
-            fields = group.fields.copy()
-            # Unconnect to prevent infinite loop
-            group.signal_unconnect(group)
-            group.destroy()
-        elif record.model_name == self.attrs['relation']:
-            fields = record.group.fields
-        parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {},
-                parent=record, parent_name=parent_name,
-                child_name=self.name,
-                context=self.context,
-                parent_datetime_field=self.attrs.get('datetime_field'))
-        if record.value.get(self.name):
-            group.record_removed.extend(record.value[self.name])
-            group.record_deleted.extend(record.value[self.name].record_deleted)
-            group.record_removed.extend(record.value[self.name].record_removed)
-        record.value[self.name] = group
-        group.fields = fields
-        group.load(value)
-        group.signal_connect(group, 'group-changed', self._group_changed)
-        group.signal_connect(group, 'group-list-changed',
-            self._group_list_changed)
-        group.signal_connect(group, 'group-cleared', self._group_cleared)
-        group.signal_connect(group, 'record-modified', self._record_modified)
-
     def get_on_change_value(self, record):
         return self.get_eval(record)
 
 
-class ReferenceField(CharField):
+class ReferenceField(Field):
 
     _default = None
 
@@ -777,9 +776,15 @@ class ReferenceField(CharField):
             return None
 
     def get(self, record):
+        if record.parent_name == self.name:
+            if record.parent:
+                return '%s,%s' % (record.group.parent.model_name,
+                    record.parent.id)
+            else:
+                return None
         if (record.value.get(self.name)
                 and record.value[self.name][0]
-                and record.value[self.name][1] >= 0):
+                and record.value[self.name][1] >= -1):
             return ','.join(map(str, record.value[self.name]))
         return None
 
@@ -791,6 +796,11 @@ class ReferenceField(CharField):
             if isinstance(ref_id, (tuple, list)):
                 ref_id, rec_name = ref_id
             else:
+                if ref_id:
+                    try:
+                        ref_id = int(ref_id)
+                    except ValueError:
+                        pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
                     rec_name = record.value.get(self.name + '.rec_name', '')
                 else:
@@ -801,6 +811,12 @@ class ReferenceField(CharField):
             force_change=force_change)
 
     def set(self, record, value):
+        if record.parent_name == self.name:
+            if record.parent:
+                value = '%s,%s' % (record.group.parent.model_name,
+                    record.parent.id)
+            else:
+                value = None
         if not value:
             record.value[self.name] = self._default
             return
@@ -820,24 +836,24 @@ class ReferenceField(CharField):
             if not rec_name and ref_id >= 0:
                 try:
                     result, = RPCExecute('model', ref_model, 'read', [ref_id],
-                        ['rec_name'], main_iteration=False)
+                        ['rec_name'])
                 except RPCException:
                     return
                 rec_name = result['rec_name']
         elif ref_model:
             rec_name = ''
         else:
-            rec_name = ref_id
+            rec_name = str(ref_id)
         record.value[self.name] = ref_model, ref_id
         record.value[self.name + '.rec_name'] = rec_name
 
 
-class BinaryField(CharField):
+class BinaryField(Field):
 
     _default = None
 
     def get(self, record):
-        result = record.value.get(self.name) or self._default
+        result = record.value.get(self.name, self._default)
         if isinstance(result, basestring):
             try:
                 with open(result, 'rb') as fp:
@@ -875,8 +891,7 @@ class BinaryField(CharField):
             context = record.context_get()
             try:
                 values, = RPCExecute('model', record.model_name, 'read',
-                    [record.id], [self.name], main_iteration=False,
-                    context=context)
+                    [record.id], [self.name], context=context)
             except RPCException:
                 return ''
             _, filename = tempfile.mkstemp(prefix='tryton_')
@@ -886,21 +901,24 @@ class BinaryField(CharField):
         return self.get(record)
 
 
-class DictField(CharField):
+class DictField(Field):
 
     _default = {}
 
+    def get_client(self, record):
+        return super(DictField, self).get_client(record) or self._default
+
     def validation_domains(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return screen_domain, screen_domain
+        return screen_domain
 
     def domain_get(self, record):
         screen_domain, attr_domain = self.domains_get(record)
-        return localize_domain(inverse_leaf(screen_domain)) + attr_domain
+        return concat(localize_domain(inverse_leaf(screen_domain)),
+            attr_domain)
 
 TYPES = {
     'char': CharField,
-    'sha': CharField,
     'float_time': FloatField,
     'integer': IntegerField,
     'biginteger': IntegerField,
