@@ -1,9 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-try:
-    from weakref import WeakSet
-except ImportError:
-    from weakrefset import WeakSet
+from weakref import WeakSet
 
 from record import Record
 from field import Field, O2MField
@@ -153,7 +150,7 @@ class Group(SignalEvent, list):
 
     def load_fields(self, fields):
         for name, attr in fields.iteritems():
-            field = Field(attr['type'])
+            field = Field.get_field(attr['type'])
             attr['name'] = name
             self.fields[name] = field(attr)
             if isinstance(self.fields[name], O2MField) \
@@ -206,7 +203,7 @@ class Group(SignalEvent, list):
         for fnct in self.on_write:
             try:
                 res += RPCExecute('model', self.model_name, fnct, ids,
-                    main_iteration=False, context=self.context)
+                    context=self.context)
             except RPCException:
                 return []
         return list({}.fromkeys(res))
@@ -395,12 +392,9 @@ class Group(SignalEvent, list):
         if len(new) and len(to_add):
             try:
                 values = RPCExecute('model', self.model_name, 'default_get',
-                    to_add.keys(), main_iteration=False, context=self.context)
+                    to_add.keys(), context=self.context)
             except RPCException:
                 return False
-            for name in to_add:
-                if name not in values:
-                    values[name] = False
             for record in new:
                 record.set_default(values, signal=signal)
 
@@ -422,7 +416,6 @@ class Group(SignalEvent, list):
                 pass
         self.clear()
         super(Group, self).destroy()
-        self.parent = None
 
     def get_by_path(self, path):
         'return record by path'
