@@ -7,6 +7,7 @@ from tryton.pyson import PYSONDecoder
 import field as fields
 from functools import reduce
 from tryton.common import RPCExecute, RPCException
+from tryton.config import CONFIG
 
 
 class Record(SignalEvent):
@@ -55,31 +56,7 @@ class Record(SignalEvent):
                         break
             else:
                 loading = self.group.fields[name].attrs.get('loading', 'eager')
-            if self in self.group and loading == 'eager':
-                idx = self.group.index(self)
-                length = len(self.group)
-                n = 1
-                while len(id2record) < 80 and (idx - n >= 0
-                        or idx + n < length) and n < 100:
-                    if idx - n >= 0:
-                        record = self.group[idx - n]
-                        if name not in record._loaded and record.id >= 0:
-                            id2record[record.id] = record
-                    if idx + n < length:
-                        record = self.group[idx + n]
-                        if name not in record._loaded and record.id >= 0:
-                            id2record[record.id] = record
-                    n += 1
-            record_context = self.context_get()
-            if loading == 'eager' and len(id2record) < 80:
-                for record in self.pool:
-                    if (name not in record._loaded
-                            and record.id >= 0
-                            and record.id not in id2record
-                            and record.context_get() == record_context):
-                        id2record[record.id] = record
-                        if len(id2record) == 80:
-                            break
+
             if loading == 'eager':
                 fnames = [fname
                     for fname, field in self.group.fields.iteritems()
@@ -93,6 +70,47 @@ class Record(SignalEvent):
             if 'rec_name' not in fnames:
                 fnames.append('rec_name')
             fnames.append('_timestamp')
+
+            record_context = self.context_get()
+            if loading == 'eager':
+                limit = CONFIG['client.limit']
+                if not self.parent:
+                    # If not a children no need to load too much
+                    limit = int(limit / len(fnames))
+
+                def filter_group(record):
+                    return name not in record._loaded and record.id >= 0
+
+                def filter_pool(record):
+                    return (filter_group(record)
+                        and record.id not in id2record
+                        and record.context_get() == record_context)
+
+                if self.parent:
+                    pool = list(self.pool)
+                else:
+                    # Don't look at the pool if it is the root
+                    pool = []
+                for group, filter_ in (
+                        (self.group, filter_group),
+                        (pool, filter_pool),
+                        ):
+                    if self in group:
+                        idx = group.index(self)
+                        length = len(group)
+                        n = 1
+                        while len(id2record) < limit and (idx - n >= 0
+                                or idx + n < length) and n < 2 * limit:
+                            if idx - n >= 0:
+                                record = group[idx - n]
+                                if filter_(record):
+                                    id2record[record.id] = record
+                            if idx + n < length:
+                                record = group[idx + n]
+                                if filter_(record):
+                                    id2record[record.id] = record
+                            n += 1
+
             ctx = record_context.copy()
             ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
                     for fname, field in self.group.fields.iteritems()
@@ -100,8 +118,7 @@ class Record(SignalEvent):
             exception = None
             try:
                 values = RPCExecute('model', self.model_name, 'read',
-                    id2record.keys(), fnames, context=ctx,
-                    main_iteration=False)
+                    id2record.keys(), fnames, context=ctx)
             except RPCException, exception:
                 values = [{'id': x} for x in id2record]
                 default_values = dict((f, None) for f in fnames)
@@ -114,6 +131,8 @@ class Record(SignalEvent):
                     record.exception = bool(exception)
                 value = id2value.get(id)
                 if record and not record.destroyed and value:
+                    for key in record.modified_fields:
+                        value.pop(key, None)
                     record.set(value, signal=False)
         return self.group.fields.get(name, False)
 
@@ -140,6 +159,13 @@ class Record(SignalEvent):
     def parent_name(self):
         return self.group.parent_name
 
+    @property
+    def root_parent(self):
+        parent = self
+        while parent.parent:
+            parent = parent.parent
+        return parent
+
     @property
     def depth(self):
         parent = self.parent
@@ -217,7 +243,7 @@ class Record(SignalEvent):
 
     def get_loaded(self, fields=None):
         if fields:
-            return set(fields) <= self._loaded
+            return set(fields) <= (self._loaded | set(self.modified_fields))
         return set(self.group.fields.keys()) == self._loaded
 
     loaded = property(get_loaded)
@@ -244,6 +270,8 @@ class Record(SignalEvent):
     def get_on_change_value(self):
         value = {}
         for name, field in self.group.fields.iteritems():
+            if name not in self._loaded and self.id >= 0:
+                continue
             value[name] = field.get_on_change_value(self)
         value['id'] = self.id
         return value
@@ -259,10 +287,12 @@ class Record(SignalEvent):
         return result
 
     def pre_validate(self):
-        values = self.get()
+        if not self.modified_fields:
+            return True
+        values = self._get_on_change_args(self.modified_fields)
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
-                main_iteration=False, context=self.context_get())
+                context=self.context_get())
         except RPCException:
             return False
         return True
@@ -273,8 +303,7 @@ class Record(SignalEvent):
                 value = self.get()
                 try:
                     res, = RPCExecute('model', self.model_name, 'create',
-                        [value], main_iteration=False,
-                        context=self.context_get())
+                        [value], context=self.context_get())
                 except RPCException:
                     return False
                 old_id = self.id
@@ -288,8 +317,7 @@ class Record(SignalEvent):
                     context['_timestamp'] = self.get_timestamp()
                     try:
                         RPCExecute('model', self.model_name, 'write',
-                            [self.id], value, main_iteration=False,
-                            context=context)
+                            [self.id], value, context=context)
                     except RPCException:
                         return False
             self._loaded.clear()
@@ -322,7 +350,7 @@ class Record(SignalEvent):
         reload_ids = list(reload_ids)
         try:
             RPCExecute('model', record.model_name, 'delete', list(record_ids),
-                main_iteration=False, context=ctx)
+                context=ctx)
         except RPCException:
             return False
         if reload_ids:
@@ -333,8 +361,7 @@ class Record(SignalEvent):
         if len(self.group.fields):
             try:
                 vals = RPCExecute('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), main_iteration=False,
-                    context=self.context_get())
+                    self.group.fields.keys(), context=self.context_get())
             except RPCException:
                 return
             if (self.parent
@@ -351,12 +378,12 @@ class Record(SignalEvent):
             if not fieldinfo.attrs.get('autocomplete'):
                 continue
             self.do_autocomplete(fieldname)
+        return vals
 
     def rec_name(self):
         try:
             return RPCExecute('model', self.model_name, 'read', [self.id],
-                ['rec_name'], main_iteration=False,
-                context=self.context_get())[0]['rec_name']
+                ['rec_name'], context=self.context_get())[0]['rec_name']
         except RPCException:
             return ''
 
@@ -367,7 +394,7 @@ class Record(SignalEvent):
             self._check_load()
         res = True
         for field_name, field in self.group.fields.iteritems():
-            if fields and field_name not in fields:
+            if fields is not None and field_name not in fields:
                 continue
             if field.get_state_attrs(self).get('readonly', False):
                 continue
@@ -393,6 +420,8 @@ class Record(SignalEvent):
         for fieldname, value in val.items():
             if fieldname not in self.group.fields:
                 continue
+            if fieldname == self.group.exclude_field:
+                continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
                 field_rec_name = fieldname + '.rec_name'
@@ -492,10 +521,11 @@ class Record(SignalEvent):
             scope = values
             for i in arg.split('.'):
                 if i not in scope:
-                    scope = False
                     break
                 scope = scope[i]
-            res[arg] = scope
+            else:
+                res[arg] = scope
+        res['id'] = self.id
         return res
 
     def on_change(self, fieldname, attr):
@@ -504,8 +534,7 @@ class Record(SignalEvent):
         args = self._get_on_change_args(attr)
         try:
             res = RPCExecute('model', self.model_name, 'on_change_' +
-                fieldname, args, main_iteration=False,
-                context=self.context_get())
+                fieldname, args, context=self.context_get())
         except RPCException:
             return
         self.set_on_change(res)
@@ -536,8 +565,7 @@ class Record(SignalEvent):
         if fieldnames:
             try:
                 result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), main_iteration=False,
-                    context=self.context_get())
+                    values, list(fieldnames), context=self.context_get())
             except RPCException:
                 return
             self.set_on_change(result)
@@ -548,7 +576,7 @@ class Record(SignalEvent):
             try:
                 result = RPCExecute('model', self.model_name,
                     'on_change_with_' + fieldname, values,
-                    main_iteration=False, context=self.context_get())
+                    context=self.context_get())
             except RPCException:
                 return
             self.group.fields[fieldname].set_on_change(self, result)
@@ -566,8 +594,7 @@ class Record(SignalEvent):
         args = self._get_on_change_args(autocomplete)
         try:
             res = RPCExecute('model', self.model_name, 'autocomplete_' +
-                fieldname, args, main_iteration=False,
-                context=self.context_get())
+                fieldname, args, context=self.context_get())
         except RPCException:
             # ensure res is a list
             res = []
@@ -582,7 +609,7 @@ class Record(SignalEvent):
                     'search_count', [
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
-                        ], main_iteration=False)
+                        ], context=self.context_get())
             except RPCException:
                 return 0
         return self.attachment_count
@@ -594,8 +621,5 @@ class Record(SignalEvent):
             if hasattr(v, 'destroy'):
                 v.destroy()
         super(Record, self).destroy()
-        self.group = None
-        self.value = None
-        self.next = None
         self.destroyed = True
         pool.remove(self)
