@@ -2,6 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
+
 from interface import WidgetInterface
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
@@ -9,6 +10,8 @@ from tryton.gui.window.win_form import WinForm
 from tryton.config import CONFIG
 import tryton.common as common
 from tryton.common import RPCExecute, RPCException
+from tryton.common.placeholder_entry import PlaceholderEntry
+from tryton.common.completion import get_completion, update_completion
 
 _ = gettext.gettext
 
@@ -35,18 +38,28 @@ class One2Many(WidgetInterface):
         tooltips = common.Tooltips()
 
         self.focus_out = True
+        self.wid_completion = None
         if attrs.get('add_remove'):
 
-            self.wid_text = gtk.Entry()
+            self.wid_text = PlaceholderEntry()
+            self.wid_text.set_placeholder_text(_('Search'))
             self.wid_text.set_property('width_chars', 13)
-            self.wid_text.connect('activate', self._sig_activate)
-            self.wid_text.connect('focus-out-event', self._focus_out)
+            self.wid_text.connect('focus-out-event',
+                lambda *a: self._focus_out())
             hbox.pack_start(self.wid_text, expand=True, fill=True)
 
+            if int(self.attrs.get('completion', 1)):
+                self.wid_completion = get_completion()
+                self.wid_completion.connect('match-selected',
+                    self._completion_match_selected)
+                self.wid_completion.connect('action-activated',
+                    self._completion_action_activated)
+                self.wid_text.set_completion(self.wid_completion)
+                self.wid_text.connect('changed', self._update_completion)
+
             self.but_add = gtk.Button()
-            tooltips.set_tip(self.but_add, _('Add'))
+            tooltips.set_tip(self.but_add, _('Add existing record'))
             self.but_add.connect('clicked', self._sig_add)
-            self.but_add.connect('enter-notify-event', self.enter)
             img_add = gtk.Image()
             img_add.set_from_stock('tryton-list-add',
                 gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -56,9 +69,9 @@ class One2Many(WidgetInterface):
             hbox.pack_start(self.but_add, expand=False, fill=False)
 
             self.but_remove = gtk.Button()
-            tooltips.set_tip(self.but_remove, _('Remove <Del>'))
+            tooltips.set_tip(self.but_remove,
+                _('Remove selected record'))
             self.but_remove.connect('clicked', self._sig_remove, True)
-            self.but_remove.connect('enter-notify-event', self.enter)
             img_remove = gtk.Image()
             img_remove.set_from_stock('tryton-list-remove',
                 gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -72,7 +85,6 @@ class One2Many(WidgetInterface):
         self.but_new = gtk.Button()
         tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
         self.but_new.connect('clicked', self._sig_new)
-        self.but_new.connect('enter-notify-event', self.enter)
         img_new = gtk.Image()
         img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_new.set_alignment(0.5, 0.5)
@@ -83,7 +95,6 @@ class One2Many(WidgetInterface):
         self.but_open = gtk.Button()
         tooltips.set_tip(self.but_open, _('Edit selected record <F2>'))
         self.but_open.connect('clicked', self._sig_edit)
-        self.but_open.connect('enter-notify-event', self.enter)
         img_open = gtk.Image()
         img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_open.set_alignment(0.5, 0.5)
@@ -94,7 +105,6 @@ class One2Many(WidgetInterface):
         self.but_del = gtk.Button()
         tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
         self.but_del.connect('clicked', self._sig_remove, False)
-        self.but_del.connect('enter-notify-event', self.enter)
         img_del = gtk.Image()
         img_del.set_from_stock('tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_del.set_alignment(0.5, 0.5)
@@ -105,7 +115,6 @@ class One2Many(WidgetInterface):
         self.but_undel = gtk.Button()
         tooltips.set_tip(self.but_undel, _('Undelete selected record <Ins>'))
         self.but_undel.connect('clicked', self._sig_undelete)
-        self.but_undel.connect('enter-notify-event', self.enter)
         img_undel = gtk.Image()
         img_undel.set_from_stock('tryton-undo', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_undel.set_alignment(0.5, 0.5)
@@ -118,7 +127,6 @@ class One2Many(WidgetInterface):
         self.but_pre = gtk.Button()
         tooltips.set_tip(self.but_pre, _('Previous'))
         self.but_pre.connect('clicked', self._sig_previous)
-        self.but_pre.connect('enter-notify-event', self.enter)
         img_pre = gtk.Image()
         img_pre.set_from_stock('tryton-go-previous',
             gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -133,7 +141,6 @@ class One2Many(WidgetInterface):
         self.but_next = gtk.Button()
         tooltips.set_tip(self.but_next, _('Next'))
         self.but_next.connect('clicked', self._sig_next)
-        self.but_next.connect('enter-notify-event', self.enter)
         img_next = gtk.Image()
         img_next.set_from_stock('tryton-go-next', gtk.ICON_SIZE_SMALL_TOOLBAR)
         img_next.set_alignment(0.5, 0.5)
@@ -146,7 +153,6 @@ class One2Many(WidgetInterface):
         but_switch = gtk.Button()
         tooltips.set_tip(but_switch, _('Switch'))
         but_switch.connect('clicked', self.switch_view)
-        but_switch.connect('enter-notify-event', self.enter)
         img_switch = gtk.Image()
         img_switch.set_from_stock('tryton-fullscreen',
             gtk.ICON_SIZE_SMALL_TOOLBAR)
@@ -172,6 +178,7 @@ class One2Many(WidgetInterface):
             view_ids=attrs.get('view_ids', '').split(','),
             views_preload=attrs.get('views', {}),
             row_activate=self._on_activate,
+            readonly=self.attrs.get('readonly', False),
             exclude_field=attrs.get('relation_field', None))
         self.screen.pre_validate = bool(int(attrs.get('pre_validate', 0)))
         self.screen.signal_connect(self, 'record-message', self._sig_label)
@@ -196,19 +203,31 @@ class One2Many(WidgetInterface):
         if (event.keyval == gtk.keysyms.F3) \
                 and self.but_new.get_property('sensitive'):
             self._sig_new(widget)
-            return False
+            return True
         if event.keyval == gtk.keysyms.F2 \
                 and widget == self.screen.widget:
             self._sig_edit(widget)
-            return False
+            return True
         if (event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete)
                 and widget == self.screen.widget
                 and self.but_del.get_property('sensitive')):
             self._sig_remove(widget)
-            return False
+            return True
         if event.keyval == gtk.keysyms.Insert and widget == self.screen.widget:
             self._sig_undelete(widget)
-            return False
+            return True
+        if self.attrs.get('add_remove'):
+            editable = self.wid_text.get_editable()
+            activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+            if not self.wid_completion:
+                activate_keys.append(gtk.keysyms.Return)
+            if (widget == self.wid_text
+                    and event.keyval in activate_keys
+                    and editable
+                    and self.wid_text.get_text()):
+                self._sig_add()
+                self.wid_text.grab_focus()
+        return False
 
     def destroy(self):
         self.screen.destroy()
@@ -289,13 +308,13 @@ class One2Many(WidgetInterface):
         if record:
             fields = self.screen.current_view.get_fields()
             if not record.validate(fields):
-                self.screen.display()
+                self.screen.display(set_cursor=True)
                 return False
             if self.screen.pre_validate and not record.pre_validate():
                 return False
         return True
 
-    def _sig_new(self, widget):
+    def _sig_new(self, widget=None):
         if not common.MODELACCESS[self.screen.model_name]['create']:
             return
         if not self._validate():
@@ -303,16 +322,18 @@ class One2Many(WidgetInterface):
         ctx = {}
         ctx.update(self.field.context_get(self.record))
         sequence = None
-        if self.screen.current_view.view_type == 'tree':
-            sequence = self.screen.current_view.widget_tree.sequence
+        for view in self.screen.views:
+            if view.view_type == 'tree':
+                sequence = view.widget_tree.sequence
+                if sequence:
+                    break
 
         def update_sequence():
             if sequence:
                 self.screen.group.set_sequence(field=sequence)
 
-        if (self.screen.current_view.view_type == 'form') \
-                or self.screen.editable_get():
-            self.screen.new(context=ctx)
+        if self.screen.current_view.editable:
+            self.screen.new()
             self.screen.current_view.widget.set_sensitive(True)
             update_sequence()
         else:
@@ -353,15 +374,7 @@ class One2Many(WidgetInterface):
     def _sig_undelete(self, button):
         self.screen.unremove()
 
-    def _sig_activate(self, *args):
-        self._sig_add()
-        self.wid_text.grab_focus()
-
-    def _focus_out(self, *args):
-        if self.wid_text.get_text():
-            self._sig_add()
-
-    def _sig_add(self, *args):
+    def _sig_add(self, *args, **kwargs):
         if not self.focus_out:
             return
         access = common.MODELACCESS[self.screen.model_name]
@@ -370,8 +383,7 @@ class One2Many(WidgetInterface):
         self.view.set_value()
         domain = self.field.domain_get(self.record)
         context = self.field.context_get(self.record)
-        domain = domain[:]
-        domain.extend(self.record.expr_eval(self.attrs.get('add_remove')))
+        domain = [domain, self.record.expr_eval(self.attrs.get('add_remove'))]
         removed_ids = self.field.get_removed_ids(self.record)
 
         self.focus_out = False
@@ -388,15 +400,22 @@ class One2Many(WidgetInterface):
             self.focus_out = True
             return False
 
+        sequence = None
+        if self.screen.current_view.view_type == 'tree':
+            sequence = self.screen.current_view.widget_tree.sequence
+
         def callback(result):
             self.focus_out = True
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
                 self.screen.display(res_id=ids[0])
+                if sequence:
+                    self.screen.group.set_sequence(field=sequence)
             self.screen.set_cursor()
             self.wid_text.set_text('')
-        if len(ids) != 1:
+
+        if len(ids) != 1 or kwargs.get('win_search', False):
             WinSearch(self.attrs['relation'], callback, sel_multi=True,
                 ids=ids, context=context, domain=domain,
                 view_ids=self.attrs.get('view_ids', '').split(','),
@@ -424,7 +443,7 @@ class One2Many(WidgetInterface):
         if field is None:
             self.screen.new_group()
             self.screen.current_record = None
-            self.screen.parent = True
+            self.screen.parent = None
             self.screen.display()
             return False
         new_group = field.get_client(record)
@@ -432,24 +451,57 @@ class One2Many(WidgetInterface):
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
             if (self.screen.current_view.view_type == 'tree') \
-                    and self.screen.editable_get():
+                    and self.screen.current_view.editable:
                 self.screen.current_record = None
             readonly = False
             domain = []
+            size_limit = None
             if record:
                 readonly = field.get_state_attrs(record).get('readonly', False)
                 domain = field.domain_get(record)
+                size_limit = record.expr_eval(self.attrs.get('size'))
             if self.screen.domain != domain:
                 self.screen.domain = domain
             if not self.screen.group.readonly and readonly:
                 self.screen.group.readonly = readonly
+            self.screen.size_limit = size_limit
         self.screen.display()
         return True
 
     def set_value(self, record, field):
-        self.screen.save_tree_state()
         self.screen.current_view.set_value()
         if self.screen.modified():  # TODO check if required
             record.modified_fields.setdefault(field.name)
             record.signal('record-modified')
         return True
+
+    def _completion_match_selected(self, completion, model, iter_):
+        record_id, = model.get(iter_, 1)
+        self.screen.load([record_id], modified=True)
+        self.wid_text.set_text('')
+        self.wid_text.grab_focus()
+
+        completion_model = self.wid_completion.get_model()
+        completion_model.clear()
+        completion_model.search_text = self.wid_text.get_text()
+        return True
+
+    def _update_completion(self, widget):
+        if self._readonly:
+            return
+        if not self.record:
+            return
+        model = self.attrs['relation']
+        domain = self.field.domain_get(self.record)
+        domain = [domain, self.record.expr_eval(self.attrs.get('add_remove'))]
+        removed_ids = self.field.get_removed_ids(self.record)
+        domain = ['OR', domain, ('id', 'in', removed_ids)]
+        update_completion(self.wid_text, self.record, self.field, model,
+            domain=domain)
+
+    def _completion_action_activated(self, completion, index):
+        if index == 0:
+            self._sig_add(win_search=True)
+            self.wid_text.grab_focus()
+        elif index == 1:
+            self._sig_new()
