@@ -26,7 +26,6 @@ class TextBox(WidgetInterface, TranslateMixin):
         self.textview.set_wrap_mode(gtk.WRAP_WORD)
         #TODO better tab solution
         self.textview.set_accepts_tab(False)
-        self.textview.connect('focus-in-event', lambda x, y: self._focus_in())
         self.textview.connect('focus-out-event',
             lambda x, y: self._focus_out())
         self.textview.connect('key-press-event', self.send_modified)
@@ -135,6 +134,8 @@ class TextBox(WidgetInterface, TranslateMixin):
         field.set_client(record, self.get_value())
 
     def set_buffer(self, value):
+        if value == self.get_value():
+            return
         buf = self.textview.get_buffer()
         buf.delete(buf.get_start_iter(), buf.get_end_iter())
         iter_start = buf.get_start_iter()
