@@ -2,14 +2,13 @@
 #this repository contains the full copyright notices and license terms.
 
 import os
-import gobject
 import tempfile
 import gtk
 import locale
 import gettext
 import webbrowser
 
-from functools import wraps
+from functools import wraps, partial
 
 from editabletree import EditableTreeView, TreeView
 from tryton.gui.window.view_form.view.interface import ParserInterface
@@ -18,7 +17,7 @@ from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
 import tryton.rpc as rpc
 from tryton.common import COLORS, node_attributes, \
-        file_selection, file_open
+        file_selection, file_open, slugify
 import tryton.common as common
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrendererdate import CellRendererDate
@@ -34,7 +33,7 @@ from tryton.common.cellrendererclickablepixbuf import \
 from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
 from tryton.common.completion import get_completion, update_completion
-from tryton.common.selection import SelectionMixin
+from tryton.common.selection import SelectionMixin, PopdownMixin
 
 _ = gettext.gettext
 
@@ -52,23 +51,23 @@ def sort_model(column, treeview, screen):
         if col != column:
             col.arrow_show = False
             col.arrow.hide()
-    screen.sort = None
+    screen.order = None
     if not column.arrow_show:
         column.arrow_show = True
         column.arrow.set(gtk.ARROW_DOWN, gtk.SHADOW_IN)
         column.arrow.show()
-        screen.sort = [(column.name, 'ASC')]
+        screen.order = [(column.name, 'ASC')]
     else:
         if column.arrow.get_property('arrow-type') == gtk.ARROW_DOWN:
             column.arrow.set(gtk.ARROW_UP, gtk.SHADOW_IN)
-            screen.sort = [(column.name, 'DESC')]
+            screen.order = [(column.name, 'DESC')]
         else:
             column.arrow_show = False
             column.arrow.hide()
     store = treeview.get_model()
     unsaved_records = [x for x in store.group if x.id < 0]
     search_string = screen.screen_container.get_text() or None
-    if screen.search_count == len(store) or unsaved_records:
+    if screen.search_count == len(store) or unsaved_records or screen.parent:
         ids = screen.search_filter(search_string=search_string, only_ids=True)
         store.sort(ids)
     else:
@@ -108,7 +107,6 @@ class ParserTree(ParserInterface):
         treeview.sequence = attrs.get('sequence', False)
         treeview.colors = attrs.get('colors', '"black"')
         treeview.keyword_open = attrs.get('keyword_open', False)
-        treeview.connect('focus', self.set_selection)
         self.treeview = treeview
         treeview.set_property('rules-hint', True)
         if not self.title:
@@ -259,6 +257,7 @@ class ParserTree(ParserInterface):
                 string = node_attrs.get('string', _('Unknown'))
                 col = gtk.TreeViewColumn(string, renderer)
                 col.name = None
+                col.set_visible(not node_attrs.get('tree_invisible', False))
 
                 label = gtk.Label(string)
                 label.show()
@@ -296,12 +295,6 @@ class ParserTree(ParserInterface):
         treeview.set_fixed_height_mode(True)
         return treeview, dict_widget, state_widgets, on_write, [], None
 
-    def set_selection(self, treeview, direction):
-        selection = treeview.get_selection()
-        if len(treeview.get_model()) and not selection.count_selected_rows():
-            selection.select_path(0)
-        return False
-
 
 class Affix(object):
 
@@ -360,12 +353,15 @@ class Affix(object):
 
 class Char(object):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
         super(Char, self).__init__()
         self.field_name = field_name
         self.model_name = model_name
         self.attrs = attrs or {}
-        self.renderer = CellRendererText()
+        if renderer is None:
+            renderer = CellRendererText
+        self.renderer = renderer()
         self.renderer.connect('editing-started', self.editing_started)
         self.treeview = treeview
 
@@ -469,19 +465,34 @@ class Char(object):
 
 class Int(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
+        if renderer is None:
+            renderer = CellRendererInteger
         super(Int, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-        self.renderer = CellRendererInteger()
-        self.renderer.connect('editing-started', self.editing_started)
+            attrs=attrs, renderer=renderer)
+        self.factor = float(attrs.get('factor', 1))
+
+    def get_textual_value(self, record):
+        if not record:
+            return ''
+        return record[self.field_name].get_client(record, factor=self.factor)
+
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, text, factor=self.factor)
+        if callback:
+            callback()
 
 
-class Boolean(Int):
+class Boolean(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
+        if renderer is None:
+            renderer = CellRendererToggle
         super(Boolean, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-        self.renderer = CellRendererToggle()
+            attrs=attrs, renderer=renderer)
         self.renderer.connect('toggled', self._sig_toggled)
 
     def _sig_toggled(self, renderer, path):
@@ -510,19 +521,17 @@ class URL(Char):
 
 class Date(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
+        if renderer is None:
+            renderer = partial(CellRendererDate, date_format())
         super(Date, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-        self.renderer = CellRendererDate(date_format())
+            attrs=attrs, renderer=renderer)
         self.renderer.connect('editing-started', self.editing_started)
 
 
 class Datetime(Date):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
-        super(Datetime, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-
     @realized
     def setter(self, column, cell, store, iter):
         super(Datetime, self).setter(column, cell, store, iter)
@@ -534,10 +543,6 @@ class Datetime(Date):
 
 class Time(Date):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
-        super(Time, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-
     @realized
     def setter(self, column, cell, store, iter):
         super(Time, self).setter(column, cell, store, iter)
@@ -547,28 +552,30 @@ class Time(Date):
         self.renderer.format = time_format
 
 
-class Float(Char):
+class Float(Int):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
+        if renderer is None:
+            renderer = CellRendererFloat
         super(Float, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-        self.renderer = CellRendererFloat()
-        self.renderer.connect('editing-started', self.editing_started)
+            attrs=attrs, renderer=renderer)
 
     @realized
     def setter(self, column, cell, store, iter):
         super(Float, self).setter(column, cell, store, iter)
         record = store.get_value(iter, 0)
         field = record[self.field_name]
-        digits = field.digits(record)
+        digits = field.digits(record, factor=self.factor)
         cell.digits = digits
 
 
 class FloatTime(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
         super(FloatTime, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
+            attrs=attrs, renderer=renderer)
         self.conv = None
         if attrs and attrs.get('float_time'):
             self.conv = rpc.CONTEXT.get(attrs['float_time'])
@@ -588,15 +595,18 @@ class FloatTime(Char):
 
 class Binary(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
-        super(Binary, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
         self.filename = attrs.get('filename')
-        self.renderer = CellRendererBinary(bool(self.filename))
+        if renderer is None:
+            renderer = partial(CellRendererBinary, bool(self.filename))
+        super(Binary, self).__init__(field_name, model_name, treeview,
+            attrs=attrs, renderer=renderer)
         self.renderer.connect('new', self.new_binary)
         self.renderer.connect('open', self.open_binary)
         self.renderer.connect('save', self.save_binary)
         self.renderer.connect('clear', self.clear_binary)
+        self.last_open_file = None
 
     def get_textual_value(self, record):
         pass
@@ -608,12 +618,36 @@ class Binary(Char):
     @realized
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
-        size = record[self.field_name].get_size(record)
+        field = record[self.field_name]
+        if hasattr(field, 'get_size'):
+            size = field.get_size(record)
+        else:
+            size = len(field.get(record))
         cell.set_property('size', common.humanize(size) if size else '')
 
+        states = ('invisible',)
+        if getattr(self.treeview, 'editable', False):
+            states = ('readonly', 'required', 'invisible')
+
+        field.state_set(record, states=states)
+        invisible = field.get_state_attrs(record).get('invisible', False)
+        cell.set_property('visible', not invisible)
+
+        if getattr(self.treeview, 'editable', False):
+            readonly = self.attrs.get('readonly',
+                field.get_state_attrs(record).get('readonly', False))
+            if invisible:
+                readonly = True
+            cell.set_property('editable', not readonly)
+
     def new_binary(self, renderer, path):
-        filename = file_selection(_('Open...'))
         record, field = self._get_record_field(path)
+        filename = ''
+        if self.last_open_file:
+            last_id, last_filename = self.last_open_file
+            if last_id == record.id:
+                filename = last_filename
+        filename = file_selection(_('Open...'), filename=filename)
         if filename:
             field.set_client(record, open(filename, 'rb').read())
             if self.filename:
@@ -626,17 +660,22 @@ class Binary(Char):
         dtemp = tempfile.mkdtemp(prefix='tryton_')
         record, field = self._get_record_field(path)
         filename_field = record.group.fields.get(self.filename)
-        filename = filename_field.get(record).replace(
-            os.sep, '_').replace(os.altsep or os.sep, '_')
+        filename = filename_field.get(record)
         if not filename:
             return
+        root, ext = os.path.splitext(filename)
+        filename = ''.join([slugify(root), os.extsep, slugify(ext)])
         file_path = os.path.join(dtemp, filename)
         with open(file_path, 'wb') as fp:
-            fp.write(field.get_data(record))
+            if hasattr(field, 'get_data'):
+                fp.write(field.get_data(record))
+            else:
+                fp.write(field.get(record))
         root, type_ = os.path.splitext(filename)
         if type_:
             type_ = type_[1:]
         file_open(file_path, type_)
+        self.last_open_file = (record.id, file_path)
 
     def save_binary(self, renderer, path):
         filename = ''
@@ -648,7 +687,10 @@ class Binary(Char):
             action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if filename:
             with open(filename, 'wb') as fp:
-                fp.write(field.get_data(record))
+                if hasattr(field, 'get_data'):
+                    fp.write(field.get_data(record))
+                else:
+                    fp.write(field.get(record))
 
     def clear_binary(self, renderer, path):
         record, field = self._get_record_field(path)
@@ -657,12 +699,12 @@ class Binary(Char):
 
 class M2O(Char):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
+        if renderer is None and int(attrs.get('completion', 1)):
+            renderer = partial(CellRendererTextCompletion, self.set_completion)
         super(M2O, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
-        if int(attrs.get('completion', 1)):
-            self.renderer = CellRendererTextCompletion(self.set_completion)
-            self.renderer.connect('editing-started', self.editing_started)
+            attrs=attrs, renderer=renderer)
 
     def value_from_text(self, record, text, callback=None):
         field = record.group.fields[self.field_name]
@@ -880,37 +922,26 @@ class M2M(O2M):
             context=context)
 
 
-class Selection(Char, SelectionMixin):
+class Selection(Char, SelectionMixin, PopdownMixin):
 
     def __init__(self, *args):
         super(Selection, self).__init__(*args)
         self.renderer = CellRendererCombo()
         self.renderer.connect('editing-started', self.editing_started)
         self.init_selection()
-        self.renderer.set_property('model', self.get_model(self.selection))
+        self.renderer.set_property('model', self.get_popdown_model(self.selection)[0])
         self.renderer.set_property('text-column', 0)
 
-    def get_model(self, selection):
-        model = gtk.ListStore(gobject.TYPE_STRING)
-        self._selection = {}
-        lst = []
-        for (value, name) in selection:
-            name = str(name)
-            lst.append(name)
-            self._selection[name] = value
-            i = model.append()
-            model.set(i, 0, name)
-        return model
-
     def get_textual_value(self, record):
         field = record[self.field_name]
         self.update_selection(record, field)
-        value = record[self.field_name].get(record)
-        return dict(self.selection).get(value, '')
+        value = field.get(record)
+        text = dict(self.selection).get(value, '')
+        if value and not text:
+            text = self.get_inactive_selection(value)
+        return text
 
     def value_from_text(self, record, text, callback=None):
-        field = record[self.field_name]
-        field.set_client(record, self._selection.get(text, False))
         if callback:
             callback()
 
@@ -919,38 +950,47 @@ class Selection(Char, SelectionMixin):
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
         field = record[self.field_name]
+
+        set_value = lambda *a: self.set_value(editable, record, field)
+        editable.child.connect('activate', set_value)
+        editable.child.connect('focus-out-event', set_value)
+        editable.connect('changed', set_value)
+
         self.update_selection(record, field)
-        model = self.get_model(self.selection)
-        editable.set_model(model)
-        # GTK 2.24 and above use a ComboBox instead of a ComboBoxEntry
-        if hasattr(editable, 'set_text_column'):
-            editable.set_text_column(0)
-        completion = gtk.EntryCompletion()
-        #Only available in PyGTK 2.6 and above.
-        if hasattr(completion, 'set_inline_selection'):
-            completion.set_inline_selection(True)
-        completion.set_model(model)
-        editable.get_child().set_completion(completion)
-        completion.set_text_column(0)
+        self.set_popdown(self.selection, editable)
+
+        value = field.get(record)
+        if not self.set_popdown_value(editable, value):
+            self.get_inactive_selection(value)
+            self.set_popdown_value(editable, value)
+        return False
+
+    def set_value(self, editable, record, field):
+        value = self.get_popdown_value(editable)
+        if 'relation' in self.attrs and value:
+            value = (value, editable.get_active_text())
+        field.set_client(record, value)
         return False
 
 
 class Reference(Char, SelectionMixin):
 
-    def __init__(self, field_name, model_name, treeview, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None,
+            renderer=None):
         super(Reference, self).__init__(field_name, model_name, treeview,
-            attrs=attrs)
+            attrs=attrs, renderer=renderer)
         self.init_selection()
-        self._selection = dict(self.selection)
 
     def get_textual_value(self, record):
-        value = record[self.field_name].get_client(record)
+        field = record[self.field_name]
+        self.update_selection(record, field)
+        value = field.get_client(record)
         if not value:
             model, name = '', ''
         else:
             model, name = value
         if model:
-            return self._selection.get(model, model) + ',' + name
+            return dict(self.selection).get(model, model) + ',' + name
         else:
             return name
 
@@ -1017,6 +1057,8 @@ class Button(object):
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
         states = record.expr_eval(self.attrs.get('states', {}))
+        if record.group.readonly or record.readonly:
+            states['readonly'] = True
         invisible = states.get('invisible', False)
         cell.set_property('visible', not invisible)
         readonly = states.get('readonly', False)
